import cml
from cml import load_knowledge

from sklearn.cluster import KMeans
from sklearn.cluster import SpectralClustering
from sklearn.feature_selection import VarianceThreshold
from sklearn.ensemble import ExtraTreesClassifier
from sklearn.tree import DecisionTreeClassifier
from sklearn.svm import SVC


def demo():

    source = cml.get_source("/home/dmt/.cml/data/double_toyset.csv")
    kmeans = KMeans(precompute_distances=False, algorithm="elkan", n_jobs=2)
    spectral = SpectralClustering(affinity='rbf', n_jobs=2)
    algorithms = [("Kme", kmeans), ("Spe", spectral)]
    constructor = cml.construction("conceptual", algorithms)

    variance = VarianceThreshold(1000)
    forest = ExtraTreesClassifier(n_estimators=250, random_state=1)
    selector = cml.feature_selection(filter_method=variance,
                                     embedded_method=forest)

    svc = SVC(gamma=2, C=1)
    tree = DecisionTreeClassifier(max_depth=5)
    algos = [("Svc", svc), ("Tre", tree)]

    reconstructor = cml.reconstruction("conceptual", algos)
    deconstructor = cml.deconstruction("conceptual", source)

    result = cml.search_knowledge(constructor,
                                  selector,
                                  reconstructor,
                                  deconstructor,
                                  stdout=True,
                                  parallel=True)
    result.dump("/tmp/lol")
    knowledge = load_knowledge("/tmp/lol")

    source = cml.get_source("/home/dmt/.cml/data/double_toyset.csv")
    deconstructor = cml.deconstruction("conceptual", source, knowledge=knowledge)
    print("Starting Next Deconstruction")
    input()
    result_2 = cml.search_knowledge(constructor,
                                    selector,
                                    reconstructor,
                                    deconstructor,
                                    stdout=True)


if __name__ == '__main__':
    demo()
