from cml.shared.settings import ConstructionSettings
from cml.shared.parameter import *

import pytest


def test_max_categories_calling():
    # arrange
    settings = ConstructionSettings()

    # assert
    assert settings.max_categories == 2


def test_max_categories_raises_type_error():
    # arrange
    settings = ConstructionSettings()

    # assert
    with pytest.raises(TypeError) as e:
        # act
        settings.max_categories = list()

    assert str(e.value) == ("Invalid type for max_categories. "
                            "Tried to redefine max_categories. "
                            "Make sure that the value is of type int or string.")


def test_max_categories_raises_value_error_during_casting():
    # arrange
    settings = ConstructionSettings()

    # assert
    with pytest.raises(ValueError) as e:
        # act
        settings.max_categories = "two"

    assert str(e.value) == ("Invalid value for max_categories. "
                            "Tried to convert string to integer. "
                            "Make sure that the value is a valid number string.")


def test_max_categories_raises_value_error_when_value_is_to_small():
    # arrange
    settings = ConstructionSettings()

    # assert
    with pytest.raises(ValueError) as e:
        # act
        settings.max_categories = 1

    assert str(e.value) == ("Invalid value for max_categories. "
                            "Tried to redefine max_categories. "
                            "Make sure that the value is greater equal two.")


def test_min_category_size_calling():
    # arrange
    settings = ConstructionSettings()

    # assert
    assert settings.min_category_size == 10


def test_min_category_size_raises_value_error_when_value_is_to_small():
    # arrange
    settings = ConstructionSettings()

    # act
    with pytest.raises(ValueError) as e:
        settings.min_category_size = 0

    # assert
    assert str(e.value) == ("Invalid value for min_category_size. "
                            "Tried to redefine min_category_size. "
                            "Make sure that the value is greater equal 1.")


def test_min_category_size_raises_value_error_during_converting():
    # arrange
    settings = ConstructionSettings()

    # act
    with pytest.raises(ValueError) as e:
        settings.min_category_size = "two"

    # assert
    assert str(e.value) == ("Invalid value for min_category_size. "
                            "Tried to convert string to integer. "
                            "Make sure that the value is a valid number string.")


def test_min_category_size_raise_type_error():
    # arrange
    settings = ConstructionSettings()

    # act
    with pytest.raises(TypeError) as e:
        settings.min_category_size = list()

    assert str(e.value) == ("Invalid type for min_category_size. "
                            "Tried to redefine min_category_size. "
                            "Make sure that the value is of type int or string.")
